# Creating HSDB Transformations Tables
# E. Schymanski, 16 May 2020


###### Directory and Package Setup #####
base_dir <- "C:/Users/emma.schymanski/ownCloud/TODOs/Interns/Jessy/HSDB/"
pc_git_tp_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/tps/"
extr_anno_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/tps/extractAnnotations.R?inline=false"
hsdb_done_dir <- paste0(base_dir,"HSDB_done/")

#hsdb_zenodo_url <- ""
##TODO: add Zenodo URL and download file automatically once available

setwd(base_dir)
#get the latest newest functions: 
download.file(extr_anno_url,paste0(base_dir,"extractAnnotations.R"))

hsdb_file <- paste0(base_dir,"Metabolism_Metabolites_HSDB_merged.csv")
file.exists(hsdb_file)

##If you don't have the latest RChemMass, reinstall:
#library(devtools)
# install_github("schymane/RChemMass", dependencies = F)

library(RChemMass)
source("extractAnnotations.R")



##### Code to create a single HSDB file given a CID #####

### This script is done in chronological order, first one would 
### create the HSDB files, then add them to "HSDB_done", then 
### re-create the transformations table, see next section. 

# Example:
query_CID <- 243

# this gets you a list of unique CIDs from HSDB, plus a download file
hsdb_out <- getPcAnno.TPcids(query_CID, hsdb_file)
hsdb_cid_file <- hsdb_out$TP_File_Name
hsdb_tp_cids <- hsdb_out$TP_Unique_CIDs

# Not all of the CIDs retrieved from HSDB are genuine TPs 
# So, you have to check in the "hsdb_cid_file" file and mark
# the information to include, before proceeding
# The functions later on expect a "_selected.csv" file ending

hsdb_done_dir <- paste0(base_dir,"HSDB_done/")
hsdb_selected_file <- paste0(base_dir,
                             sub(".csv","_selected.csv",basename(hsdb_cid_file)))
file.exists(hsdb_selected_file)
file.copy(hsdb_selected_file,paste0(hsdb_done_dir,basename(hsdb_selected_file)),
          overwrite=F)


###### Code to create new HSDB Transformation Tables ######

# # these are set as default, pasted here only for info 
# hsdb_desc <- paste0("HSDB is a toxicology database that focuses on the ",
#                     "toxicology of potentially hazardous chemicals. See ",
#                     "https://pubchem.ncbi.nlm.nih.gov/source/11933")
# dataset_DOI <- "10.5281/zenodo.3827487"
# dataset_desc <- "Transformation Products extracted from HSDB content in PubChem, validated by LCSB-ECI"


#base_dir <- "C:/Users/emma.schymanski/ownCloud/TODOs/Interns/Jessy/HSDB/"
pc_git_tp_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/tps/"
base_dir <- paste0(pc_git_tp_dir,"HSDB/")
hsdb_done_dir <- paste0(base_dir,"HSDB_done/")
setwd(base_dir)

hsdb_structure_file <- paste0(base_dir,"S68_HSDBTPS_StructureInfoOnly.csv")
file.exists(hsdb_structure_file)

createTransFile.hsdb(hsdb_done_dir,hsdb_structure_file)

hsdb_info <- read.csv("S68_HSDBTPS_TransformationTable.csv", stringsAsFactors = F)
hsdb_refs <- unique(hsdb_info[,c(12,13)])
write.csv(hsdb_refs,"S68_HSDBTPS_RefInfo.csv",row.names = F)

#read in modified refs file and map up:
hsdb_refs <- read.csv("S68_HSDBTPS_RefInfo_sort.csv", stringsAsFactors = F)
hsdb_info <- read.csv("S68_HSDBTPS_TransformationTable.csv", stringsAsFactors = F,
                      colClasses = "character")
i <- 1
for (i in 1:length(hsdb_refs$Evidence_Description)) {
  ref_string <- hsdb_refs$Evidence_Description[i]
  ref_indexes <- grep(ref_string, hsdb_info$Evidence_Description,fixed=T)
  hsdb_info$Evidence_ID[ref_indexes] <- hsdb_refs$Evidence_ID[i]
}
write.csv(hsdb_info,"S68_HSDBTPS_TransformationTable.csv",row.names = F,na="")
