# Getting ChEMBL Transformations 
# E. Schymanski, 27/5/2021
# Data in PubChem worked up by Tiejun

source('~/GitHub/pubchem/annotations/tps/extractAnnotations.R')

# testing existing function
n_pages <- getPcAnno.TotalPages("NORMAN Suspect List Exchange","Transformations")
# answer = 5

# testing new annotations
n_pages <- getPcAnno.TotalPages("ChEMBL","Transformations")
# answer = 2

# This retrieves only the CIDs from the first page
getPcAnno.Transformations(source = 'NORMAN Suspect List Exchange',heading = 'Transformations')
# This will retrieve all as a list of CIDs

#source <- "NORMAN Suspect List Exchange"
source <- "ChEMBL"
heading <- "Transformations"
n_pages <- getPcAnno.TotalPages(source, heading)
for (i in 1:n_pages) {
  if (i == 1) {
    cids <- getPcAnno.Transformations(source, heading, page=i)
  } else {
    cids_px <- getPcAnno.Transformations(source, heading, page=i) 
    cids <- c(cids,cids_px)
    }
}
length(cids)

# May 2021 this was 4730 CIDs for NORMAN-SLE
# May 2021 this was 1418 for ChEMBL

getwd()
loc_trans_dir <- "C:/DATA/PubChem/Transformations/"
setwd("C:/DATA/PubChem/Transformations/")
ChEMBL_dir <- paste0(loc_trans_dir,"ChEMBL/")
if(!dir.exists(ChEMBL_dir)) {
  dir.create(ChEMBL_dir)
}

setwd(ChEMBL_dir)

#getPcCand.trans(1017)
# from ChEMBL:
# https://pubchem.ncbi.nlm.nih.gov/sdq/sdqagent.cgi?infmt=json&outfmt=csv&query={%22download%22:%22*%22,%22collection%22:%22chemblmetabolism%22,%22where%22:{%22ands%22:[{%22cid%22:%221983%22}]},%22order%22:[%22relevancescore,desc%22],%22start%22:1,%22limit%22:10000000,%22downloadfilename%22:%22CID_1983_chemblmetabolism%22}
getPcCand.trans(92223,dataset = "chemblmetabolism")

#for (i in 1:length(cids)) {
for (i in 1046:length(cids)) {
    getPcCand.trans(cids[i],dataset = "chemblmetabolism")
}


## Now merge all the files together ... 

trans_files <- list.files(ChEMBL_dir, full.names = T)
length(trans_files)
# #remove the NA file if present ...
# trans_files <- trans_files[-grep("_NA_",trans_files)]
# length(trans_files)

trans_data <- read.csv(trans_files[1],stringsAsFactors = F)
trans_data2 <- read.csv(trans_files[2],stringsAsFactors = F)
trans_data_all <- merge(trans_data,trans_data2,all.x=TRUE,all.y=TRUE)
for (i in 3:length(trans_files)) {
  file_name <- trans_files[i]
  # NA_test <- grep("_NA_",file_name,fixed=TRUE)
  # if (length(NA_test)==0) {
  trans_data2 <- read.csv(file_name)
  trans_data_all <- merge(trans_data_all,trans_data2,all.x=TRUE,all.y=TRUE)
  # }
}
all_trans_file <- paste0(loc_trans_dir,"ChEMBL_all_transformations.csv")
# # trim two lead cols from the dataset
# write.csv(trans_data_all[,c(-1,-2)],all_trans_file,row.names = F)
# this should now be fixed, so no trimming
write.csv(trans_data_all,all_trans_file,row.names = F)


# this will need to be merged into NORMAN-SLE transformations, but will 
# need some header translation ... 

# NORMAN-SLE
SLE_cols <- c("cid","predecessor", "predecessorcid", "transformation", "successor",
              "successorcid", "evidencedoi", "evidenceref", "sourcecomment",
              "sourcecommentfull", "datasetdoi", "datasetref", "enzyme", 
              "biosystem", "cmpdname")

ChEMBL_cols <- c("metid", "substrate", "metabolite", "substratechemblid",
                 "metabolitechemblid", "substratecid", "metabolitecid",
                 "metconversion", "enzyme", "enzymechemblid", "cids",
                 "protacxns", "geneids", "pmids", "dois")


# merge the two datasets

SLE_trans <- read.csv("NORMAN-SLE_all_transformations.csv",stringsAsFactors = F)
ChEMBL_trans <- read.csv("ChEMBL_all_transformations.csv",stringsAsFactors = F)

# keep only essential columns:
SLE_trans <- SLE_trans[,c(2:6)]
ChEMBL_trans <- ChEMBL_trans[,c(2,6,8,3,7)]
merged_colnames <- SLE_cols[2:6]
colnames(ChEMBL_trans) <- merged_colnames
length(SLE_trans$predecessorcid) # 8084, matches manual eval
length(ChEMBL_trans$predecessorcid) #1748, matches manual eval

merged_trans <- merge(SLE_trans,ChEMBL_trans,all=T)
length(merged_trans$predecessorcid) #9832 - lots of duplicates
length(unique(merged_trans)$predecessorcid) # 5393, but possible duplicates due to names

merged_trans$cid_to_cid <- ""
i <- 1
for (i in 1:length(merged_trans$cid_to_cid)) {
  merged_trans$cid_to_cid[i] <- paste0(merged_trans$predecessorcid[i],">>",
                                       merged_trans$successorcid[i])
}
length(unique(merged_trans$cid_to_cid)) #4900, matches manual eval

all_CIDs <- unique(na.omit(c(merged_trans$predecessorcid,merged_trans$successorcid)))
#which(is.na(all_CIDs)) # added omit ... none now. 
length(all_CIDs) #5966 - matches manual eval 

# retrieve the information using tips from Rick
library(webchem)
library(data.table)
selected_properties <- c("MolecularFormula","ExactMass","XlogP",
                         "CanonicalSMILES","IsomericSMILES",
                         "InChI","InChIKey","Title","IUPACName")
# retrieve info with webchem
CID_info_all <- as.data.table(webchem::pc_prop(all_CIDs, selected_properties))

# add IKFB
CID_info_all$IKFB <- strtrim(CID_info_all$InChIKey,14)

# output
write.csv(CID_info_all,"Transformations_CID_Info_all.csv",row.names = F)


# ## superceded code ##
# # retrieve the information needed in chunks of 100
# n_calls <- ceiling(length(all_CIDs)/100)
# # test:
# call_url <- "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/cid/1,2,3,4,5/property/MolecularFormula,MolecularWeight,CanonicalSMILES/CSV"
# download.file(call_url,"test_CSV.csv")
# 
# # new dir for CSV files:
# trans_cid_dir <- paste0(loc_trans_dir,"CID_info")
# if(!dir.exists(trans_cid_dir)) {
#   dir.create(trans_cid_dir)
# }
# 
# base_URL <- "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/cid/"
# URL_cids <- paste(all_CIDs[1:100],collapse=",")
# mid_URL <- "/property/"
# selected_properties <- c("MolecularFormula","ExactMass",
#                          "CanonicalSMILES","IsomericSMILES",
#                          "InChI","InChIKey","Title","IUPACName")
# URL_properties <- paste(selected_properties,collapse=",")
# end_URL <- "/CSV"
# full_URL <- paste0(base_URL,URL_cids,mid_URL,URL_properties,end_URL)
# 
# download.file(full_URL,paste0(trans_cid_dir,"/test_CSV.csv"))
# 
# i <- 1
# 
# for (i in 1:n_calls) {
#   if (i < n_calls) {
#     call_range <- c((i*100-99),(i*100))
#   } else {
#     call_range <- c((i*100-99),length(all_CIDs))
#   }
#   URL_cids <- paste(all_CIDs[call_range[1]:call_range[2]],collapse = ",")
#   full_URL <- paste0(base_URL,URL_cids,mid_URL,URL_properties,end_URL)
#   cid_info_filename <- paste0("CID_Info_call_",i,"_range_",call_range[1],
#                               "_to_",call_range[2],".csv")
#   download.file(full_URL,paste0(trans_cid_dir,"/",cid_info_filename))
#   
# }
# 
# #  now merge
# CID_files <- list.files(trans_cid_dir,full.names = T)
# CID_info_one <- read.csv(CID_files[1],stringsAsFactors = F)
# CID_info_two <- read.csv(CID_files[2],stringsAsFactors = F)
# CID_info_all <- merge(CID_info_one,CID_info_two,all=T)
# 
# for (j in 3:length(CID_files)) {
#   CID_info_two <- read.csv(CID_files[j],stringsAsFactors = F)
#   CID_info_all <- merge(CID_info_all,CID_info_two,all=T)
# }
# length(CID_info_all$CID) #5966, retained all
# 
# # add IKFB
# CID_info_all$IKFB <- strtrim(CID_info_all$InChIKey,14)
# 
# 
# write.csv(CID_info_all,"Transformations_CID_Info_all.csv",row.names = F)

# now, uniquify the merged Transformations with new CID info
trans_info <- unique(merged_trans[,c(2,5,6)])
length(trans_info$cid_to_cid) #4900, same as manual eval

trans_info$IK_to_IK <- ""
trans_info$IKFB_to_IKFB <- ""
trans_info$ReactionSMILES <- ""
trans_info$predecessorEM <- ""
trans_info$successorEM <- ""
trans_info$MassDiff <- ""
trans_info$XlogPDiff <- ""

# have to remove successors and predecessors that are "NA"
NA_succ <- which(is.na(trans_info$successorcid))
trans_info <- trans_info[-NA_succ,]
NA_pred <- which(is.na(trans_info$predecessorcid))
trans_info <- trans_info[-NA_pred,]
length(trans_info$predecessorcid) # now 4885

i <- 1
for (i in 1:length(trans_info$predecessorcid)) {
  pred_CID <- trans_info$predecessorcid[i]
  succ_CID <- trans_info$successorcid[i]
  i_pred_CID <- which(pred_CID==CID_info_all$CID)
  #i_pred_CID <- grep(pred_CID,CID_info_all$CID,fixed = T)
  if (length(i_pred_CID) >1) {
    print(paste("Matching predecessor CIDs indexes:", paste(i_pred_CID,collapse = ",")))
  }
  i_succ_CID <- which(succ_CID==CID_info_all$CID)
  #i_succ_CID <- grep(succ_CID,CID_info_all$CID,fixed = T)
  if (length(i_succ_CID) >1) {
    print(paste("Matching predecessor CIDs indexes:", paste(i_succ_CID,collapse = ",")))
  }
  pred_IK <- CID_info_all$InChIKey[i_pred_CID]
  succ_IK <- CID_info_all$InChIKey[i_succ_CID]
  pred_SMILES <- CID_info_all$IsomericSMILES[i_pred_CID]
  succ_SMILES <- CID_info_all$IsomericSMILES[i_succ_CID]
  pred_EM <- as.numeric(CID_info_all$ExactMass[i_pred_CID])
  succ_EM <- as.numeric(CID_info_all$ExactMass[i_succ_CID])
  pred_XlogP <- as.numeric(CID_info_all$XLogP[i_pred_CID])
  succ_XlogP <- as.numeric(CID_info_all$XLogP[i_succ_CID])
  
  IK_to_IK <- paste0(pred_IK,">>",succ_IK)
  IKFB_to_IKFB <- paste0(strtrim(pred_IK,14),">>",strtrim(succ_IK,14))
  ReactionSMILES <- paste0(pred_SMILES,">>",succ_SMILES)
  MassDiff <- succ_EM - pred_EM
  XlogPDiff <- succ_XlogP-pred_XlogP
  
  trans_info$IK_to_IK[i] <- IK_to_IK
  trans_info$IKFB_to_IKFB[i] <- IKFB_to_IKFB
  trans_info$ReactionSMILES[i] <- ReactionSMILES
  trans_info$predecessorEM[i] <- pred_EM
  trans_info$successorEM[i] <- succ_EM
  trans_info$MassDiff[i] <- MassDiff
  trans_info$XlogPDiff[i] <- XlogPDiff
}

write.csv(trans_info,"Transformations_Rxn_Info_all.csv",row.names = F)

# now some stats
length(unique(trans_info$IKFB_to_IKFB)) #4624 <= this is slightly more than manual eval
# but this is confirmed taking this out into excel ... also 4624. 
# look at the XlogPs
range(as.numeric(trans_info$XlogPDiff),na.rm = T)
range(as.numeric(trans_info$MassDiff))


# now look at how many unique transformations
mass_diffs <- as.character(round(as.numeric(trans_info$MassDiff),digits=4))
unique_mass_diffs <- unique(mass_diffs)
length(unique_mass_diffs)
mass_diff_count <- vector("numeric",length=length(unique_mass_diffs))

i <- 1

for (i in 1:length(mass_diff_count)) {
  #  mass_diff_count[i] <- length(grep(unique_mass_diffs[i],mass_diffs,fixed=TRUE))
  mass_diff_count[i] <- length(which(mass_diffs == unique_mass_diffs[i]))
}

tp_mass_diffs <- as.data.frame(cbind(unique_mass_diffs,mass_diff_count))
#colnames(tp_mass_diffs) <- c("Mass_difference","Count")

# now look at them in order
o <- order(mass_diff_count,decreasing = T)
tp_mass_diffs <- as.data.frame(cbind(as.numeric(unique_mass_diffs[o]),
                                     as.numeric(mass_diff_count[o])))
colnames(tp_mass_diffs) <- c("MassDiff","Count")

#write.csv(tp_mass_diffs,"TP_mass_diff_MetXBioDB.csv",row.names = F)
write.csv(tp_mass_diffs,"Transformations_MassDiffs_all.csv",row.names = F)
write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "Transformations_MassDiffs_ge5.csv",row.names = F)


# match with previous SLOD file
tp_mass_diffs$Reaction <- ""
tp_mass_diffs$Formula_gained <- ""
tp_mass_diffs$Formula_lost <- ""
tp_mass_diffs$SMARTS <- ""

# mass_diff_anno_URL <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/slod/transformations_mass_diffs_ge5_wAnnotation.csv"
# download.file(mass_diff_anno_URL,destfile = "slod_transformations_mass_diffs_ge5_wAnnotation.csv")
# slod_data <- read.csv("slod_transformations_mass_diffs_ge5_wAnnotation.csv",
#                            stringsAsFactors = F)
# # use updated file in future
mass_diff_anno_URL <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/slod/Transformations_MassDiffs_ge5_wAnnotation.csv"
download.file(mass_diff_anno_URL,destfile = "slod_Transformations_MassDiffs_ge5_wAnnotation.csv")
slod_data <- read.csv("slod_Transformations_MassDiffs_ge5_wAnnotation.csv",
                      stringsAsFactors = F)


i <- 1
for (i in 1:length(tp_mass_diffs$MassDiff)) {
  massdiff_i <- tp_mass_diffs$MassDiff[i]
  #i_metx <- grep(massdiff_i,MetXBioDB_data$Mass_difference,fixed=T,value=T)
  i_metx <- which(slod_data$MassDiff==massdiff_i)
  if (length(i_metx)>0) {
    tp_mass_diffs$Reaction[i] <- slod_data$Reaction[i_metx]
    tp_mass_diffs$Formula_gained[i] <- slod_data$Formula_gained[i_metx]
    tp_mass_diffs$Formula_lost[i] <- slod_data$Formula_lost[i_metx]
    tp_mass_diffs$SMARTS[i] <- slod_data$SMARTS[i_metx]
  }
}

write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "Transformations_MassDiffs_ge5_wAnnotation.csv",row.names = F)






# when satisfied with the file, copy to GitLab ... 

