# MassBank records to MassBank compounds for PubChem deposition
# E. Schymanski, 5 March 2020.
# Based off record index file provided by T. Schulze. 

# set up dir etc
mb_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/massbank_eu/"
setwd(mb_dir)
library(RChemMass)

mb_records <- read.csv(file="MassBank_Records_Release_2019.12.csv",stringsAsFactors = F)

mb_cmpds <- unique(mb_records[,c(2,3,4,5,6,7,8,9)])

#strsplit(mb_records$NAME[1],"|",fixed=T)

mb_cmpds$NAME2 <- ""
mb_cmpds$NAME3 <- ""
mb_cmpds$NAME4 <- ""
mb_cmpds$FirstAccession <- ""
name_string <- ""
mb_cmpds$URL <- ""
mb_url_string <- "https://massbank.eu/MassBank/Result.jsp?inchikey="

i <- 2
for (i in 1:length(mb_cmpds$NAME)) {
  name_string <- strsplit(mb_cmpds$NAME[i],"|",fixed=T)[[1]]
  mb_cmpds$NAME2[i] <- name_string[1]
  mb_cmpds$NAME3[i] <- name_string[2]
  mb_cmpds$NAME4[i] <- name_string[3]
  mb_cmpds$FirstAccession[i] <- mb_records$ACCESSION[as.numeric(row.names(mb_cmpds[i,]))]
  mb_cmpds$URL[i] <- paste0(mb_url_string,mb_cmpds$InChIKey[i])
}

#Tips from Jeff:
#For structure data, if you have CID, you may not need smiles or inchi string 
#(correct me if I am wrong, @Shoemaker, Ben (NIH/NLM/NCBI) [E]), 
#If there is no CID, the use smiles first, if no smiles and no cid, 
# use inchi string.
# I recommend to use inchikey as your ext-id since 
# different accession number may have same structures

# get unique InChIKeys
InChIKey <- unique(mb_cmpds$InChIKey)

mb_unique_cmpds <- as.data.frame(InChIKey,stringsAsFactors = F)
mb_unique_cmpds$SMILES <- "" #later PUBCHEM_EXT_DATASOURCE_SMILES
mb_unique_cmpds$InChI <- "" #later PUBCHEM_EXT_DATASOURCE_INCHI
mb_unique_cmpds$CAS <- "" #later PUBCHEM_SUBSTANCE_SYNONYM
mb_unique_cmpds$PC_CID <- "" #later PUBCHEM_EXT_DATASOURCE_CID
mb_unique_cmpds$Name1 <- "" # later PUBCHEM_SUBSTANCE_SYNONYM
mb_unique_cmpds$Name2 <- "" # later PUBCHEM_SUBSTANCE_SYNONYM
mb_unique_cmpds$Name3 <- "" # later PUBCHEM_SUBSTANCE_SYNONYM
mb_unique_cmpds$URL <- "" #later PUBCHEM_EXT_SUBSTANCE_URL

write.csv(mb_unique_cmpds,"MassBank_UniqueCmpds_Release_2019_12.csv",row.names = F)
#retrieve CIDs through ID Exchange
# we have 278 InChIKeys without CIDs (one NA removed ... ) now sorted to the top
mb_unique_cmpds <- read.csv("MassBank_UniqueCmpds_Release_2019_12.csv", stringsAsFactors = F)

i <- 1
for (i in 1:278) {
  IK_curr <- as.character(mb_unique_cmpds$InChIKey[i])
  IK_index <- grep(IK_curr,mb_cmpds$InChIKey,fixed=TRUE)
  #mb_unique_cmpds$PC_CID <- 
  mb_unique_cmpds$SMILES[i] <- mb_cmpds$SMILES[IK_index[1]]
  mb_unique_cmpds$InChI[i] <- mb_cmpds$StdInChI[IK_index[1]]
  mb_unique_cmpds$CAS[i] <- CAS_fix(mb_cmpds$CAS_RN[IK_index[1]])
  mb_unique_cmpds$PC_CID[i] <- mb_cmpds$PUBCHEM_CID[IK_index[1]]
  mb_unique_cmpds$Name1[i] <- mb_cmpds$NAME2[IK_index[1]]
  mb_unique_cmpds$Name2[i] <- mb_cmpds$NAME3[IK_index[1]]
  mb_unique_cmpds$Name3[i] <- mb_cmpds$NAME4[IK_index[1]]
  mb_unique_cmpds$URL[i] <- paste0(mb_url_string,IK_curr)
}

for (i in 279:length(mb_unique_cmpds$InChIKey)) {
  IK_curr <- as.character(mb_unique_cmpds$InChIKey[i])
  IK_index <- grep(IK_curr,mb_cmpds$InChIKey,fixed=TRUE)
  mb_unique_cmpds$CAS[i] <- mb_cmpds$CAS_RN[IK_index[1]]
  mb_unique_cmpds$Name1[i] <- mb_cmpds$NAME2[IK_index[1]]
  mb_unique_cmpds$Name2[i] <- mb_cmpds$NAME3[IK_index[1]]
  mb_unique_cmpds$Name3[i] <- mb_cmpds$NAME4[IK_index[1]]
  mb_unique_cmpds$URL[i] <- paste0(mb_url_string,IK_curr)
}


# change column names to PubChem fields
# https://pubchemdocs.ncbi.nlm.nih.gov/substance-tag-names
colnames_uni <- colnames(mb_unique_cmpds)

#Col 1: InChIKey => Ext Reg ID
colnames_uni[1] <- "PUBCHEM_EXT_DATASOURCE_REGID"
#Col 2: SMILES
colnames_uni[2] <- "PUBCHEM_EXT_DATASOURCE_SMILES"
#Col 3: InChI
colnames_uni[3] <- "PUBCHEM_EXT_DATASOURCE_INCHI"
#Col 4: CAS
colnames_uni[4] <- "PUBCHEM_SUBSTANCE_SYNONYM"
#Col 5: PubChemCID from MB.EU
colnames_uni[5] <- "PUBCHEM_EXT_DATASOURCE_CID"
#Col 6, 7, 8: names
colnames_uni[6] <- "PUBCHEM_SUBSTANCE_SYNONYM"
colnames_uni[7] <- "PUBCHEM_SUBSTANCE_SYNONYM"
colnames_uni[8] <- "PUBCHEM_SUBSTANCE_SYNONYM"
#Col 9: URL
colnames_uni[9] <- "PUBCHEM_EXT_SUBSTANCE_URL"

colnames(mb_unique_cmpds) <- colnames_uni

write.csv(mb_unique_cmpds,"MassBank_UniqueCmpds_Release_2019_12.csv",
          row.names = F,na="")

# this file passes submission validation!