# Evaluation Script for PubChemLite
# Emma Schymanski, 18 Nov 2019
# Revised 11 May 2020 and many dates since
# Preparing for upload 28 Oct 2020 & submission 10 Nov. 

###### Set up packages & folders ######

# Set up packages
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install()

# ##If you don't have the latest RChemMass, reinstall:
# library(devtools)
# install_github("MassBank/RMassBank",dependencies=F)
# install_github("schymane/RChemMass", dependencies = F)
# install_github("schymane/ReSOLUTION", dependencies = F)
library(devtools)
library(RMassBank)
library(ReSOLUTION)
library(RChemMass)

# set up folders
# by default this will work off and create subfolders 
# in the current directory
base_dir <- getwd()
# # Please adjust path as necessary for your chosen system
setwd(base_dir)

# This script requires that all necessary CSV files are saved here:
db_dir <- paste0(base_dir, "/MetFrag_CSVs/")
# To download necessary files, see "PCLite_eval_prepMetFrag.R"
# Please adjust path as necessary for your chosen system

# This script requires that the MetFrag jar is saved here:
metfrag_dir <- paste0(base_dir, "/MetFrag/")
# To download necessary files, see "PCLite_eval_prepMetFrag.R"
# Please adjust path as necessary for your chosen system

# get user_PCLite_eval.R file
user_file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/user_PCLite_eval.R?inline=false"
download.file(user_file_url,paste0(base_dir,"user_PCLite_eval.R"))
source(paste0(base_dir,"user_PCLite_eval.R"))
# get PCLite_eval_support.R file
user_file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/PCLite_eval_support.R?inline=false"
download.file(user_file_url,paste0(base_dir,"PCLite_eval_support.R"))
source(paste0(base_dir,"PCLite_eval_support.R"))
# get extractAnnotations.R file
extr_anno_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/tps/extractAnnotations.R?inline=false"
download.file(extr_anno_url,paste0(base_dir,"extractAnnotations.R"))
source(paste0(base_dir,"extractAnnotations.R"))

# get the PCLite Benchmarking file needed
zenodo_url <- "https://zenodo.org/record/4146957/files/"
file_url <- paste0(zenodo_url, "PCLite_Benchmark_wMSMS_info.csv")
download.file(file_url, paste0(base_dir,"PCLite_Benchmark_wMSMS_info.csv"))


##### MetFrag Preparation ######

# To prepare files for MetFrag, please see "PCLite_eval_prepMetFrag.R"
file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/PCLite_eval_prepMetFrag.R"
download.file(file_url, paste0(base_dir,"PCLite_eval_prepMetFrag.R"))
# please open script and run sections as needed
# this includes MS/MS and mapping files needed for sections below. 


##### Evaluation for various PCLites - CASMI Set ######
# 18Nov2019 tier0
# set up files 
msms_dir <- paste0(base_dir,"MSMS/CASMI_2016/")
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier0.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir,"solutions_casmi2016_cat2and3.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_18Nov19_CASMI2016_wFPSum"
pre2020 <- TRUE

runPCLiteEval.CASMI(cmpd_file,localDB,run_name,metfrag_dir,msms_dir,pre2020=T,
                    base_dir=base_dir,verbose = F)

# 18Nov2019 tier1
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier1.csv")
file.exists(localDB)

runPCLiteEval.CASMI(cmpd_file,localDB,run_name="tier1_18Nov19_CASMI2016_wFPSum",
                    metfrag_dir,msms_dir,pre2020 = T,base_dir = base_dir,
                    verbose=T)

# 14Jan2020 tier0
localDB <- paste0(db_dir,"PubChemLite_14Jan2020_tier0.csv")
file.exists(localDB)
run_name <- "tier0_14Jan2020_CASMI2016_wAnnoTypeCount"

runPCLiteEval.CASMI(cmpd_file,localDB,run_name, metfrag_dir,msms_dir,
                    pre2020 = F,base_dir = base_dir,verbose=T)
# In case you need to check anything without rerunning the calcs ... 
runPCLiteEval.CASMI(cmpd_file,localDB,run_name, metfrag_dir,msms_dir,
                    pre2020 = F,base_dir = base_dir,verbose=T,runMetFrag = F)

# 14Jan2020 tier1
localDB <- paste0(db_dir,"PubChemLite_14Jan2020_tier1.csv")
file.exists(localDB)
run_name <- "tier1_14Jan2020_CASMI2016_wAnnoTypeCount"

runPCLiteEval.CASMI(cmpd_file,localDB,run_name, metfrag_dir,msms_dir,
                    pre2020 = F,base_dir = base_dir,verbose=T)
# Test functions from the GitLab export now, check without rerunning the calcs ... 
runPCLiteEval.CASMI(cmpd_file,localDB,run_name, metfrag_dir,msms_dir,
                    pre2020 = F,base_dir = base_dir,verbose=T,runMetFrag = F)


##### Evaluation for various PCLites - MetFrag Relaunched Sets ######

##### Evaluation for various PCLites - MetFragRL - EA Set ######
# 18Nov2019 tier0
# set up files 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/EA/")
dir.exists(msms_dir)
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier0.csv")
file.exists(localDB)
# set up results
run_name <- "tier0_18Nov19_MetFragRL_EA_wFPSum"
pre2020 <- TRUE

# The problem with this set is that there was no input file ... 
# so create one based on the directory listing
cmpd_info <- mk_cmpd_info.EA(msms_dir)

# set up the rest of the MetFrag columns
cmpd_info <- user_mk_cmpd_info(cmpd_info)


runPCLiteEval.EA(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                 pre2020=T,base_dir=base_dir,verbose=T,runMetFrag = T)
# Test w/out rerunning MetFrag: 
# runPCLiteEval.EA(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
#                  pre2020=T,base_dir=base_dir,verbose=T,runMetFrag = F)



##### Evaluation for various PCLites+MetFragRL: EQEx, EQExPlus, UF Sets ######

# CHIN-D-15-00088 EQEx, EQExPlus and UF sets

# ##EQEx data

# EQEx data
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier0.csv")
file.exists(localDB)
# set up results
run_name <- "tier0_18Nov19_MetFragRL_EQEx_wFPSum"
pre2020 <- TRUE

# create the compound info 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/EQEx/")
cmpd_info <- mk_cmpd_info.EQtoUF(msms_dir)
cmpd_info <- user_mk_cmpd_info(cmpd_info)

# # to just test, run this, otherwise comment out subsetting: 
cmpd_info <- cmpd_info[1:3,]
# run MetFrag evaluation
runPCLiteEval.EQtoUF(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,EQ=T,
                     pre2020=pre2020,base_dir=base_dir,verbose=T,runMetFrag = T)


# #EQExPlus data
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier0.csv")
file.exists(localDB)
# set up results
run_name <- "tier0_18Nov19_MetFragRL_EQExPlus_wFPSum"
pre2020 <- TRUE

# create the compound info 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/EQExPlus/")
cmpd_info <- mk_cmpd_info.EQtoUF(msms_dir)
cmpd_info <- user_mk_cmpd_info(cmpd_info)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.EQtoUF(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,EQ=T,
                     pre2020=pre2020,base_dir=base_dir,verbose=T,runMetFrag = T)


# UF data
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier0.csv")
file.exists(localDB)
# set up results
run_name <- "tier0_18Nov19_MetFragRL_UF_wFPSum"
pre2020 <- TRUE

# create the compound info 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/UF/")
cmpd_info <- mk_cmpd_info.EQtoUF(msms_dir)
cmpd_info <- user_mk_cmpd_info(cmpd_info)

# # to just test, run this, otherwise comment out subsetting: 
cmpd_info <- cmpd_info[1:3,]
# run MetFrag evaluation
runPCLiteEval.EQtoUF(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,EQ=F,
                     pre2020=pre2020,base_dir=base_dir,verbose=T,runMetFrag = T)


##### Evaluation for CompTox - CASMI Set ######
# set up files 
msms_dir <- paste0(base_dir,"MSMS/CASMI_2016/")
localDB <- paste0(db_dir,"CompTox_07March19_SelectMetaData.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir,"solutions_casmi2016_cat2and3.csv")
file.exists(cmpd_file)
# set up results
run_name <- "CompTox_CASMI2016"

# read in compound info
cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runCompToxEval.CASMI(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                     base_dir=base_dir,verbose = T, runMetFrag = T)

##### Evaluation for CompTox+MetFragRL: EA, EQEx, EQExPlus, UF Sets ######

## EA
# set up files 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/EA/")
localDB <- paste0(db_dir,"CompTox_07March19_SelectMetaData.csv")
file.exists(localDB)
# set up results
run_name <- "CompTox_MetFragRL_EA"
cmpd_info <- mk_cmpd_info.EA(msms_dir)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runCompToxEval.EA(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                  base_dir=base_dir,verbose = T)


## EQ
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/EQEx/")
localDB <- paste0(db_dir,"CompTox_07March19_SelectMetaData.csv")
file.exists(localDB)

# set up results
run_name <- "CompTox_MetFragRL_EQEx"
cmpd_info <- mk_cmpd_info.EQtoUF(msms_dir)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runCompToxEval.EQtoUF(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,EQ=T,
                      base_dir=base_dir,verbose = T)

## EQExPlus
# # set up files 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/EQExPlus/")
localDB <- paste0(db_dir,"CompTox_07March19_SelectMetaData.csv")
file.exists(localDB)

# set up results
run_name <- "CompTox_MetFragRL_EQExPlus"
cmpd_info <- mk_cmpd_info.EQtoUF(msms_dir)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runCompToxEval.EQtoUF(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,EQ=T,
                      base_dir=base_dir,verbose = T)


## UFZ
# # set up files 
msms_dir <- paste0(base_dir,"MSMS/MetFragRL/UF/")
localDB <- paste0(db_dir,"CompTox_07March19_SelectMetaData.csv")
file.exists(localDB)

# set up results
run_name <- "CompTox_MetFragRL_UF"
cmpd_info <- mk_cmpd_info.EQtoUF(msms_dir)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runCompToxEval.EQtoUF(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,EQ=F,
                      base_dir=base_dir,verbose = T)


#### Evaluation for Benchmarking set - PubChemLite ####

## First step (run once only): creating the benchmarking file
## see separate script ...PubChemLiteEval_addMSMStoBenchmark.R

## Evaluations

### 14 Jan, tier 1 
localDB <- paste0(db_dir,"PubChemLite_14Jan2020_tier1.csv")
file.exists(localDB)
# post MSMS being added to file:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier1_14Jan2020_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS=TRUE,
                 pre2020=FALSE,base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

## 14 Jan, tier0
localDB <- paste0(db_dir,"PubChemLite_14Jan2020_tier0.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_14Jan2020_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS=TRUE,
                 pre2020=FALSE,base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

# 18 Nov 2019, tier0
localDB <- paste0(db_dir,"PubChemLite_18Nov2019_tier0.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_18Nov2019_MergedBenchmark"
pre2020 <- TRUE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

## 22 May 2020, tier0
localDB <- paste0(db_dir,"PubChemLite_tier0_20200522.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_22May2020_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

# 12 June 2020, tier0
localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # testing briefly:
# runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
#                  base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)
# full run
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)


# 12 June 2020, tier0 with formula not exact mass!
# note that the interim versions (May, June) didn't have elements removed
# properly, hence "mod" file is available and used for June.
# If used on the May file, this will probably result in errors
localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_MolForm"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- FALSE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass)

# full run
setwd(base_dir)
# runPCLiteEval.BM(cmpd_info[837:length(cmpd_info$PCIDs),],
#                  localDB,run_name,metfrag_dir,useMSMS,pre2020,
#                  base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
#                  useExactMass = useExactMass,start_i=836)
# runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
#                  base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
#                  useExactMass = useExactMass)
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass)


# 14 Jan 2020, tier0 with formula not exact mass!
localDB <- paste0(db_dir,"PubChemLite_14Jan2020_tier0.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "tier0_14Jan2020_MergedBenchmark_MolForm"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- FALSE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass)
# full run
# cmpd_info$MolecularFormula[836] <- "C22H23N6O7S2"
# runPCLiteEval.BM(cmpd_info[837:length(cmpd_info$PCIDs),],
#                  localDB,run_name,metfrag_dir,useMSMS,pre2020,
#                  base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE, 
#                  useExactMass = useExactMass)

runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE, 
                 useExactMass = useExactMass)

### Now try with Agrochem subset of the benchmark...

localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)

# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_AgroChemOnly"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- TRUE #Have switched back to using Exact Mass for now. 

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)

# Now, select only for the AgroChem
agro_ikfbs <- read.delim("benchmark_agchem_ikfb.lst", header = F,
                         stringsAsFactors = F)
#agchem_matchs <- pmatch(agro_ikfbs[,1],cmpd_info$IK_FB)
#agchem_matches <- agro_ikfbs[,1] %in% cmpd_info$IK_FB
agchem_matches <- cmpd_info$IK_FB %in% agro_ikfbs[,1]
agchem_i <- which(agchem_matches)
cmpd_info <- cmpd_info[agchem_i,]
# add the extra columns
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,extra_terms = c("agro"))

# adjusting the terms to be Agrochem specific
# Note if these are left empty ("") in the function they default to "normal"
UDS_terms <- "PubMed_Count,Patent_Count,AgroChemInfo"
UDS_weights <- ",1,1,1"

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=c("agro"),
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#rerun again to test without rerunning MetFrag results ... 
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=FALSE,
                 useExactMass = useExactMass, extra_terms=c("agro"),
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#now try full run ... 
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=c("agro"),
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)


### Now try with DrugMedic subset of the benchmark...

localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)

# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_DrugMedicOnly"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- TRUE #Have switched back to using Exact Mass for now. 

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)

# Now, select only for the DrugMedic Entries
subset_file <- paste0(base_dir, "benchmark_DrugMedicInfo_ikfb.tab")
subset_ikfbs <- read.delim(subset_file, header = F,
                         stringsAsFactors = F)
subset_matches <- cmpd_info$IK_FB %in% subset_ikfbs$V2
subset_i <- which(subset_matches)
cmpd_info <- cmpd_info[subset_i,]
# add the extra columns
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,extra_terms = c("drug"))

# adjusting the terms to be DrugMedic specific
# Note if these are left empty ("") in the function they default to "normal"
UDS_terms <- "PubMed_Count,Patent_Count,DrugMedicInfo"
UDS_weights <- ",1,1,1"

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=c("drug"),
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#now try full run ... 
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=c("drug"),
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)


### Now try with DrugMedic subset of the benchmark...but with the 
# PharmacoInfo scoring term instead of DrugMedicInfo scoring term

localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)

# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_DrugMedic_wPharmInfScore"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- TRUE #Have switched back to using Exact Mass for now. 

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)

# Now, select only for the DrugMedic Entries
subset_file <- paste0(base_dir, "benchmark_DrugMedicInfo_ikfb.tab")
subset_ikfbs <- read.delim(subset_file, header = F,
                           stringsAsFactors = F)
subset_matches <- cmpd_info$IK_FB %in% subset_ikfbs$V2
subset_i <- which(subset_matches)
cmpd_info <- cmpd_info[subset_i,]
# add the extra columns
extra_terms <- c("pharma")
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,extra_terms = extra_terms)

# adjusting the terms to be DrugMedic specific - try PharmacoInfo score
# Note if these are left empty ("") in the function they default to "normal"
UDS_terms <- "PubMed_Count,Patent_Count,PharmacoInfo"
UDS_weights <- ",1,1,1"

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#now try full run ... 
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)


### Now try with ToxicityInfo subset of the benchmark...
# (which is almost the entire benchmark ... 708 CIDs)

localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)

# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_ToxInfoOnly"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- TRUE #Have switched back to using Exact Mass for now. 

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)

# Now, select only for the DrugMedic Entries
subset_file <- paste0(base_dir,"benchmark_ToxicityInfo_ikfb.tab")
subset_ikfbs <- read.delim(subset_file, header = F,
                           stringsAsFactors = F)
subset_matches <- cmpd_info$IK_FB %in% subset_ikfbs$V2
subset_i <- which(subset_matches)
cmpd_info <- cmpd_info[subset_i,]
# add the extra columns
extra_terms <- c("toxicity") #this adds extra terms to the table
#but still need the terms adjusted below ... 
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,extra_terms = extra_terms)

# adjusting the terms to be Tox specific
# Note if these are left empty ("") in the function they default to "normal"
UDS_terms <- "PubMed_Count,Patent_Count,ToxicityInfo"
UDS_weights <- ",1,1,1"

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#now try full run ... 
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)


### Now try with KnownUse as a scoring term ...
# ... and subset the benchmark afterwards...

localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)

# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_KnownUseOnly"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- TRUE #Have switched back to using Exact Mass for now. 

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)

# Now, select only for the KnownUse Entries
subset_file <- paste0(base_dir,"benchmark_KnownUse_ikfb.tab")
subset_ikfbs <- read.delim(subset_file, header = F,
                           stringsAsFactors = F)
subset_matches <- cmpd_info$IK_FB %in% subset_ikfbs$V2
subset_i <- which(subset_matches)
# subset this later once we have all info ... 
#cmpd_info <- cmpd_info[subset_i,]

# add the extra columns
extra_terms <- c("use") #this adds extra terms to the table
#but still need the terms adjusted below ... 
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,extra_terms = extra_terms)

# adjusting the terms to be Tox specific
# Note if these are left empty ("") in the function they default to "normal"
UDS_terms <- "PubMed_Count,Patent_Count,KnownUse"
UDS_weights <- ",1,1,1"

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#now try full run ... 
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)

cmpd_info <- read.csv("MetFragResultSummaryPubChemLite_tier0_12Jun2020_MergedBenchmark_KnownUseOnly.csv")
cmpd_info <- cmpd_info[subset_i,]


#### PubChemLite with Patents and Lit only

localDB <- paste0(db_dir,"PubChemLite_tier0_20200612_mod.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)

# set up results
run_name <- "tier0_12Jun2020_MergedBenchmark_PatLitOnly"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- TRUE #Have switched back to using Exact Mass for now. 

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)


# add the extra columns
extra_terms <- c("none") #this adds extra terms to the table
#but still need the terms adjusted below ... 
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,extra_terms = extra_terms)

# adjusting the terms to be Pat Lit only specific
# Note if these are left empty ("") in the function they default to "normal"
UDS_terms <- "PubMed_Count,Patent_Count"
UDS_weights <- ",1,1"

# testing briefly:
runPCLiteEval.BM(cmpd_info[1:3,],localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)
#now try full run ... 
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass = useExactMass, extra_terms=extra_terms,
                 UDS_terms = UDS_terms,UDS_weights = UDS_weights)

cmpd_info <- read.csv("MetFragResultSummaryPubChemLite_tier0_12Jun2020_MergedBenchmark_KnownUseOnly.csv")
cmpd_info <- cmpd_info[subset_i,]




# CompTox
localDB <- paste0(db_dir,"CompTox_07March19_SelectMetaData.csv")
file.exists(localDB)
# post MSMS:
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "CompTox_MergedBenchmark"
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- mk_cmpd_info.CompTox(cmpd_info,skipMSMS = T)

# runCompToxEval.BM(cmpd_info[1:5,],localDB,run_name,metfrag_dir,useMSMS=TRUE,
#                   base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

runCompToxEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS=TRUE,
                  base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)


##### testing new PubChemLite Exposomics #####

## 31 Oct 2020, exposomics version
localDB <- paste0(db_dir,"PubChemLite_31Oct2020_exposomics.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "exposomics_31Oct2020_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
#cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

# test using molecular formula

run_name <- "exposomics_31Oct2020_MergedBenchmark_MolForm"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- FALSE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
# for mol form, this includes previous problematic entries ... 
cmpd_info <- cmpd_info[c(1:5,835:837),]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 useExactMass=useExactMass)


#####Jan 2021 versions ####

## 1 Jan 2021, exposomics version
localDB <- paste0(db_dir,"PubChemLite_exposomics_20210101.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "exposomics_01Jan21_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
#cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)

#now try with extra term disease
localDB <- paste0(db_dir,"PubChemLite_exposomics_20210101.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "exposomics_01Jan21_MergedBM_Disease"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T, extra_terms = "disease")

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 extra_terms = "disease")



##### CCSbase Exposomics File (1 Jan 2021) #####

# file from: https://zenodo.org/record/4456208 

localDB <- paste0(db_dir,"PubChemLite_01Jan2021_exposomics_CCSbase.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "exposomics_CCSbase_01Jan21_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)
cmpd_info <- mk_cmpd_info.CCSbase(cmpd_info,adducts=c("[M+H]+","[M-H]-"))

# # to just test, run this, otherwise comment out the subsetting: 
#cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                 CCSbase_adducts=c("[M+H]+","[M-H]-"))


CCS_csv <- "MetFragResultSummaryPubChemLite_exposomics_CCSbase_01Jan21_MergedBenchmark.csv"

##### CCSbase + XlopP Exposomics File (1 Jan 2021) #####

localDB <- paste0(db_dir,"PubChemLite_01Jan2021_exposomics_CCSbase_XlogP3.csv")
#note this file has too many missing XlogP3s but the XlogP file has missing entries
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "exposomics_CCSbase_XlogP_01Jan21_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE
XlogP <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T,XlogP = XlogP)
cmpd_info <- mk_cmpd_info.CCSbase(cmpd_info,adducts=c("[M+H]+","[M-H]-"))

# # to just test, run this, otherwise comment out the subsetting: 
cmpd_info <- cmpd_info[1:5,]
# run MetFrag evaluation
runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                 base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,XlogP=XlogP,
                 CCSbase_adducts=c("[M+H]+","[M-H]-"))


CCS_csv <- "MetFragResultSummaryPubChemLite_exposomics_CCSbase_XlogP3_01Jan21_MergedBenchmark.csv"



##### Supporter Functions #####
# Ported to PCLite_eval_support.R


