## Copyright (C) 2020 by University of Luxembourg

## Usage:
##
## Rscript [--vanilla] add_ccs2csv.R <input ccs csv> <input metfrag csv> <output metfrag csv>

require(data.table)
require(assertthat)
require(yaml)
USAGE_STRING <- "Rscript [--vanilla] add_ccs2csv.R <CCS input file> <input metfrag csv> <output metfrag csv>"

SEL_ADDUCTS <- NULL # Change to adducts needed, if a subset of adducts
                    # desired. Otherwise, all shall be returned.

ccs_file_adduct_names <- c("[M+H]+",
                 "[M+Na]+",
                 "[M+NH4]+",
                 "[M+K]+",
                 "[M-H]-",
                 "[M+Na-2H]-",
                 "[M]+",
                 "[M]-")

new_ccs_adduct_names <- paste0("pred_",ccs_file_adduct_names)

args <- commandArgs(trailingOnly = T)
fn_ccs <- args[[1]]
fn_mf_csv <- args[[2]]
fn_mf_res_csv <- args[[3]]


message("Input CCS file: ",fn_ccs)
message("Input Metfrag CSV file: ", fn_mf_csv)
message("Output Metfrag CSV file: ", fn_mf_res_csv)

dt_ccs <- data.table::fread(fn_ccs,sep=',')

dt_mf_csv <- data.table::fread(fn_mf_csv,sep=',')


data.table::setnames(dt_ccs,"smi","SMILES",skip_absent = T)
data.table::setnames(dt_ccs,"related_cid","Identifier",skip_absent = T)

ccs_cols <- c("Identifier","SMILES",ccs_file_adduct_names)
res <- dt_ccs[,..ccs_cols][dt_mf_csv,on="Identifier",nomatch=NULL]

## idsmi <- dt_mf_csv[,.(Identifier,SMILES)]
## res <- res[idsmi,on="Identifier",nomatch=NA]


uniq_smiles_merged <- res[,unique(SMILES)]
uniq_smiles_pcl <- dt_mf_csv[,unique(SMILES)]

diff <- setdiff(uniq_smiles_pcl,uniq_smiles_merged)
setnames(res,ccs_file_adduct_names,new_ccs_adduct_names)
data.table::fwrite(res,file=fn_mf_res_csv,sep=",",quote = "auto",na = "")
message("Number of CCS rows: ",NROW(dt_ccs))
message("Number of MF rows: ", NROW(dt_mf_csv))
message("Unique SMILES entries in pure PCL: ", length(uniq_smiles_pcl))

message("Number of MF-w-CCS rows: ", NROW(res))
message("Unique nonempty SMILES entries in PCL with CCS: ", length(uniq_smiles_merged))

message("Number-of-rows delta: ", NROW(dt_mf_csv) - NROW(res))
message("Number of SMILES in pure PCL with missing CCS: ", length(diff))


## Obsolete?
## ## Turn adduct into a number of columns.
## dt_tr_ccs <- data.table::dcast(dt_ccs, SMILES ~ adduct, value.var = "ccs")
## message("Number of dcast CCS rows: ",NROW(dt_tr_ccs))

## ## Wrangle names.
## add_nm <- grep("\\[.+",colnames(dt_tr_ccs),val=T)
## message("Adducts: ",add_nm)

## new_add_nm <- paste("pred_CCS_A2",add_nm,sep="_")
## data.table::setnames(dt_tr_ccs,add_nm,new_add_nm)
## mf_nms <- colnames(dt_mf_csv)

## sel_add_cols <- if (!is.null(SEL_ADDUCTS))
##                     paste0("pred_CCS_A2_",SEL_ADDUCTS) else new_add_nm

## all_nms <- c(mf_nms,sel_add_cols)

## res <- dt_tr_ccs[dt_mf_csv,on="SMILES",nomatch=NA][,..all_nms]
## message("Number of res rows: ",NROW(res))
## data.table::fwrite(res,file=fn_mf_res_csv,sep=",",quote = "auto",na = "")



