## Copyright (C) 2020 by University of Luxembourg

## Usage:
##
## Rscript [--vanilla] add_ccs2csv.R <input ccs csv> <input metfrag csv> <output metfrag csv>

require(data.table)
require(assertthat)
require(yaml)
USAGE_STRING <- "Rscript [--vanilla] add_ccs2csv.R <CCS input file> <input metfrag csv> <output metfrag csv>"

SEL_ADDUCTS <- NULL # Change to adducts needed, if a subset of adducts
                    # desired. Otherwise, all shall be returned.

ccs_file_adduct_names <- c("[M+H]+",
                 "[M+Na]+",
                 "[M+NH4]+",
                 "[M+K]+",
                 "[M-H]-",
                 "[M+Na-2H]-",
                 "[M]+",
                 "[M]-")

new_ccs_adduct_names <- paste0("pred_",ccs_file_adduct_names)

args <- commandArgs(trailingOnly = T)
fn_ccs <- args[[1]]
fn_mf_csv <- args[[2]]
fn_mf_res_csv <- args[[3]]


message("Input CCS file: ",fn_ccs)
message("Input Metfrag CSV file: ", fn_mf_csv)
message("Output Metfrag CSV file: ", fn_mf_res_csv)

dt_ccs <- data.table::fread(fn_ccs,sep=',')
dt_mf_csv <- data.table::fread(fn_mf_csv,sep=',')
data.table::setnames(dt_ccs,"smi","SMILES",skip_absent = T)
data.table::setnames(dt_ccs,"related_cid","Identifier",skip_absent = T)

res <- data.table::copy(dt_mf_csv)
ccs_cols <- c("Identifier","SMILES",ccs_file_adduct_names)
rhs=paste0("i.",ccs_file_adduct_names)
eval(bquote(res[dt_ccs,(new_ccs_adduct_names):=list(..(lapply(rhs,as.symbol))), on="Identifier"],splice=T))

res <- res[!is.na(`pred_[M+H]+`)]
data.table::fwrite(res,file=fn_mf_res_csv,sep=",",quote = "auto",na = "")

unsmimf <- dt_mf_csv[,unique(SMILES)]
unsmires <- res[,unique(SMILES)]
message("NROW(dt_mf_csv): ",NROW(dt_mf_csv))
message("Unique SMILES(dt_mf_csv): ",length(unsmimf))
message("NROW(res): ", NROW(res))
message("Unique SMILES(res): ", length(unsmires))
message("Delta NROW: ", NROW(dt_mf_csv) - NROW(res))
message("Delta SMILES: ", length(unsmimf) - length(unsmires))
