## Copyright (C) 2020 by University of Luxembourg

## Usage:
##
## Rscript [--vanilla] add_ccs2csv.R <input ccs csv> <input metfrag csv> <output metfrag csv>



# require(data.table)
# require(assertthat)
# USAGE_STRING <- "Rscript [--vanilla] add_ccs2csv.R <input ccs csv> <input metfrag csv> <output metfrag csv>"
# args <- commandArgs(trailingOnly = T)
# assertthat::assert_that(length(args) == 3, msg = paste("Not all arguments provided. Usage:\n",USAGE_STRING))
# fn_ccs <- args[[1]]
# fn_mf_csv <- args[[2]]
# fn_mf_res_csv <- args[[3]]
library(data.table)
library(assertthat)
base_dir <- "C:/CompoundDBs/MetFrag_CSVs/"
setwd(base_dir)

SEL_ADDUCTS <- NULL # Change to adducts needed, if a subset of adducts
# needed. Otherwise, all shall be returned. 

#fn_ccs <- paste0(base_dir,"PubChemLite_CCS_update_02-2021.csv")
#adding Sunghwan's new calcs (newer CCS model, run at PubChem)
fn_ccs <- paste0(base_dir,"PCL_CCS_from_sunghwan.csv")
fn_mf_csv <- paste0(base_dir,"PubChemLite_exposomics_20210101.csv")
fn_mf_res_csv <- paste0(base_dir,"PubChemLite_01Jan2021_exposomics_CCSbase_Sunghwan.csv")


message("Input CCS file: ",fn_ccs)
message("Input Metfrag CSV file: ", fn_mf_csv)
message("Output Metfrag CSV file: ", fn_mf_res_csv)

dt_ccs <- data.table::fread(fn_ccs,sep=',')
message("Number of CCS rows: ",NROW(dt_ccs))
dt_mf_csv <- data.table::fread(fn_mf_csv,sep=',')
message("Number of MF rows: ", NROW(dt_mf_csv))


data.table::setnames(dt_ccs,"smi","SMILES",skip_absent = T)

## Turn adduct into a number of columns.
dt_tr_ccs <- data.table::dcast(dt_ccs, SMILES ~ adduct, value.var = "ccs")
message("Number of dcast CCS rows: ",NROW(dt_tr_ccs))

## Wrangle names.
add_nm <- grep("\\[.+",colnames(dt_tr_ccs),val=T)
message("Adducts: ",add_nm)

new_add_nm <- paste("pred_CCS_A2",add_nm,sep="_")
data.table::setnames(dt_tr_ccs,add_nm,new_add_nm)
mf_nms <- colnames(dt_mf_csv)

sel_add_cols <- if (!is.null(SEL_ADDUCTS))
  paste0("pred_CCS_A2_",SEL_ADDUCTS) else new_add_nm

all_nms <- c(mf_nms,sel_add_cols)

# if NULL, entries without CCS are removed. If NA, they are retained with ""
#res <- dt_tr_ccs[dt_mf_csv,on="SMILES",nomatch=NULL][,..all_nms]
res <- dt_tr_ccs[dt_mf_csv,on="SMILES",nomatch=NA][,..all_nms]
message("Number of res rows: ",NROW(res))


data.table::fwrite(res,file=fn_mf_res_csv,sep=",",quote = "auto",na = "")




# data.table::setnames(dt_ccs,c("adduct","smi","name","ccs","mz"),c("pred_CCS_A2_adduct","SMILES","CompoundName","pred_CCS_A2","pred_CCS_A2_mz"),skip_absent = T)
# mf_nms <- colnames(dt_mf_csv)
# all_nms <- c(mf_nms,"pred_CCS_A2_adduct","pred_CCS_A2_mz","pred_CCS_A2")
# 
# res <- dt_ccs[dt_mf_csv,on="SMILES",nomatch=NULL][,..all_nms]
# ind_na <- res[is.na(pred_CCS_A2),which=T]
# if (length(ind_na) > 0) warning(paste("There are NA CCS values at indices:\n",
#                                       paste(ind_na,collapse = ",")))
# message("Number of res rows: ",NROW(res))
# 
# 
# data.table::fwrite(res,file=fn_mf_res_csv,sep=",",quote = "auto")

